a:link       { color: #FF8000; text-decoration: none; }
a:visited    { color: #FFAD5B; text-decoration: none; }
html         { color: #5C5C5C; text-decoration: none; }

.quickview a:link       { color: #FF8000; text-decoration: none; }
.quickview a:visited    { color: #FFAD5B; text-decoration: none; }
.quickview html         { color: #5C5C5C; text-decoration: none; }

.alert a:link		{ color: red; text-decoration: none; }
.alert a:visited	{ color: red; text-decoration: none; }
.alert a:html		{ color: red; text-decoration: none; }

body {
	font-family: 	Verdana,Arial,Helvetica,Sans Serif;
	font-size:	10pt;
}

th {
	background-color: #ff8000;
	text-align:	  left;
}

table#accounting {

	align:center;
	background-color:#ffffff;
	border-collapse: collapse;
	font-size: 10pt;
	width:     760;
}

table#plans {
	
	padding: 10px;
        background-color:#ffffff;
        border-collapse: collapse;
        font-size: 10pt;

}

table#error {
	font-size:	10pt;
}

.toprow {

	background-color: #FF8000;
	font-size:	  10pt;
}

.status {
}

.function {

	background-color: #ffffff;
	font-weight: 	bold;
	font-size:	10pt;
	align: 		center;
}

.plan {

	background-color: #FFffff;
	font-weight: 	bold;
	color:		#ff8000;
	font-size:	10pt;
	width:		110;

}

.heading {
	background-color: #ffff99;
}

.whitespace {
	
	background-color: #ffffff;
	color:		  #ffffff;
}

.quickview {
	font-size:	12pt;
}

.searchbox {
	maxlength:	30;
}

.info {
	font-weight:	  bold;
}

.error-text {
	text-align:	left;
}	

.alert {
	color: red;
}
