<html>
<head>
 <title>ISP Transaction Document</title>
 <link rel="stylesheet" type="text/css" href="/templates/isp_accounting.css">
 <SCRIPT type="text/javascript" src="<TMPL_VAR js_lib>">
 </SCRIPT>
</head>

<body>

<div align=center>

<TMPL_IF is_devel_system>
		<br><br>
		<font size=4 color=red><b>-------------------- Development Platform --------------------</b></font>
		<br><br><br>
</TMPL_IF>

<table>
	<tr>
		<td><img src="/graphics/image_logo.gif"></td>
	</tr>
</table>

</div>

<br>
<div align=center>
<br>
