<input type=hidden name=cc value=1>
<table id="accounting" border=1 cellspacing=0>

	<tr>
    	<th colspan=3>Credit Card Information</th>
  	</tr>

	<tr>
		<td colspan=5>&nbsp;</td>
	</tr>	
	
	<tr>
		<td><b>Credit Card Number</b></td>
		<td><b>Expiry</b></td>
		<td><b>Card Holder Name</b></td>
	</tr>

	<tr>
		<td><input type=text size=20 name=ccn></td>
		<td><input type=text size=5  name=cce></td>
		<td><input type=text size=35 name=chn></td>
	</tr>
