<br><br>

<table id="accounting" border=1 cellspacing=0>

 <tr>
  <th colspan=4>Client Information</th>
 </tr>

 <tr>
   <td colspan=4 align=right><b>[ <a href="<TMPL_VAR display_client_details_link>">DETAIL</a> ] &nbsp&nbsp [ <a href="<TMPL_VAR display_edit_client_link>">EDIT</a> ]&nbsp&nbsp</b></td>
 </tr>

 <tr>
  <td class=info colspan=2>Address</td>
  <td class=info>Home Phone:</td>
  <td><TMPL_VAR home_phone></td>
 </tr>

 <tr>
  <td colspan=2><TMPL_VAR name></td>
  <td class=info>Work Phone:</td>
  <td><TMPL_VAR work_phone></td>
 </tr>

 <tr>
  <td colspan=2><TMPL_VAR street></td>
  <td class=info>Fax:</td>
  <td><TMPL_VAR fax></td>
 </tr>

 <tr>
  <td colspan=2><TMPL_VAR city>, <TMPL_VAR province> <TMPL_VAR zip></td>
  <td></td>
  <td></td>
 </tr>

</table>
