
Account Name/Client ID/Username: [%username%]

Our time logs indicate that during the month of [%date_string%], you used
[%hours_used%] hours on the Internet. Your current plan is [%plan_name%]
which provides you with [%hours_allowed%] hours of access per month. Each
additional hour costs $[%over_rate%].

The balance owing is as follows:

....INVOICE.... [%invoice_number%]...........

  [%hours_used%] hours used 
- [%hours_allowed%] hours included in plan
------
  [%hours_over%] hours over limit
x $[%over_rate%] per additional hour
------
  $[%sub_total%] subtotal
+ $[%tax%] Tax
------
$[%total_cost%] Balance Owing

This is an automated message.
