package Business::ISP::ISP;

use strict;
use warnings;
no warnings 'redefine';

our $VERSION = '0.13_02';

=head1 NAME

Business::ISP:: - An *SP billing/client accounting system.

=head1 DESCRIPTION

This is just a test upload so I can figure out some particular intricacies of
CPAN :)

=head1 AUTHOR

Steve Bertrand, C<< <steveb at cpan.org> >>

=head1 COPYRIGHT

Copyright 2012 Steve Bertrand, all rights reserved.

=head1 LICENSE

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.


=cut

1;
