<html>
<head>
 <link rel="stylesheet" type="text/css" href="/templates/isp_accounting.css">
 <SCRIPT type="text/javascript" src="<TMPL_VAR js_lib>">
 </SCRIPT>
</head>

<body>
<div align=center>

<TMPL_IF is_devel_system>
	<br>
	<TMPL_VAR version><br><br>
	<font size=4 color=red><b>-------------------- Development Platform --------------------</b></font>
	<br><br>
</TMPL_IF>

<br>




