CREATE TABLE plans (
  id mediumint(12) NOT NULL auto_increment,
  plan_status varchar(7) default NULL,
  username varchar(45) NOT NULL default '',
  login_name varchar(45) default NULL,
  password varchar(20) default NULL,
  server varchar(60) NOT NULL default '',
  email varchar(60) default NULL,
  dob varchar(12) default NULL,
  last_update date default NULL,
  plan varchar(15) default NULL,
  description varchar(60) default NULL,
  rate decimal(6,2) default NULL,
  hours smallint(6) default NULL,
  over_rate decimal(6,2) default NULL,
  billing_period varchar(9) default NULL,
  expires date default NULL,
  started date default NULL,
  pap_date tinyint(4) default NULL,
  next_billing_date date default NULL,
  pap_method varchar(25) default NULL,
  billing_method varchar(22) default NULL,
  os varchar(8) default NULL,
  dsl_number varchar(13),  
  comment varchar(255) default NULL,
  hours_balance decimal(7,2) not null default 0,
  classification varchar(24) default NULL,
  PRIMARY KEY  (id,username)
) TYPE=MyISAM;
